<?php

namespace Wino;

class CheckImportSessionProgressCommand {
  public const ACTION = 'wino-check-import-session-progress';

  /** @return void */
  public static function initialize() {
    addAjaxAction(self::ACTION, AjaxErrorHandler::notice(), function () {
      return self::handle();
    });
  }

  /** @return AjaxResponse */
  public static function handle() {
    FormHandling::verifyPostNonce(self::ACTION);
    Auth::verifyAdminCapability();

    $translator = LocaleTranslator::fromConfiguration();

    $log = InMemoryLog::empty();
    $response = AjaxResponse::from($log);

    sleep(2);

    $import = ImportSession::persisted(DataScope::none());
    $response->replaceDomById(
      AdminPage::PROGRESS_LABEL_PAGE_TITLE_SUFFIX_SPAN_ID,
      AdminPage::progressLabelTitleSuffixSpan($import, $translator)
    );
    StartImportSessionCommand::actionToggleToAjaxResponse($response, $import, $translator);

    if ($import !== null && $import->isPending())
    $response->callAction(RunImportSessionCommand::ACTION, AjaxErrorHandler::console());

    if ($import !== null && $import->preventConcurrentImports())
    $response->callAction(self::ACTION, AjaxErrorHandler::notice());

    $report = ImportSession::savedReport();
    if ($report !== null) writeDataImportLogToAjaxResponse($report, $response, $translator);
    ImportSession::deleteReport();

    return $response;
  }

  /** @return Render */
  public static function ajaxAction(?ImportSession $import) {
    $report = ImportSession::savedReport();
    $callOnStartUp = $report !== null || ImportSession::importPreventConcurrency($import);
    return ajaxActionRender(self::ACTION, $callOnStartUp);
  }
}
