<?php

namespace Wino;

interface DeleteDataCommandTranslator extends DataImportLogEntryTranslator {
  /** @return string */
  public function nothingToDeleteLabel();

  /** @return string */
  public function deleteDataCommandLabel();

  /** @return string */
  public function deleteDataCommandConfirmationMessage();
}

class DeleteDataCommand {
  public const ACTION = 'wino-delete-data';

  /** @return string */
  public static function label(DeleteDataCommandTranslator $translator) {
    return $translator->deleteDataCommandLabel();
  }

  /** @return string */
  public static function confirmationMessage(DeleteDataCommandTranslator $translator) {
    return $translator->deleteDataCommandConfirmationMessage();
  }

  /** @return void */
  public static function initialize() {
    addAjaxAction(self::ACTION, AjaxErrorHandler::notice(), function () {
      return self::handle();
    });
  }

  /** @return AjaxResponse */
  public static function handle() {
    FormHandling::verifyPostNonce(self::ACTION);
    Auth::verifyDebugMode();
    Auth::verifyAdminCapability();

    $context = ImportContext::fromConfiguration();
    $translator = LocaleTranslator::fromConfiguration();

    $log = InMemoryLog::empty();
    $importLog = DataImportLog::empty();
    $response = AjaxResponse::from($log);

    set_time_limit(0);

    deleteAllWinoProducts($context, $importLog);
    deleteAllWinoTaxClasses($context, $importLog);
    deleteAllWinoCategories($context, $importLog);

    writeDataImportLogToAjaxResponse($importLog, $response, $translator);
    $response->enableAction(self::ACTION, self::label($translator));
    return $response;
  }

  /** @return Render */
  public static function ajaxButtonWithConfirmation(DeleteDataCommandTranslator $translator) {
    return ajaxButtonRender(
      self::label($translator),
      self::ACTION,
      ConfigurationFieldMode::adminDebug()
        ->confirm(self::confirmationMessage($translator))
        ->singleUse()
    );
  }
}
