<?php

namespace Wino;

interface FlushImportSessionCommandTranslator extends StartImportSessionCommandTranslator {
  /** @return string */
  public function flushImportSessionCommandLabel();

  /** @return string */
  public function importSessionFlushed();

  /** @return string */
  public function noImportSessionFound();
}

class FlushImportSessionCommand {
  public const ACTION = 'wino-flush-import-session';

  /** @return string */
  public static function label(FlushImportSessionCommandTranslator $translator) {
    return $translator->flushImportSessionCommandLabel();
  }

  /** @return void */
  public static function initialize() {
    addAjaxAction(self::ACTION, AjaxErrorHandler::notice(), function () {
      return self::handle();
    });
  }

  /** @return AjaxResponse */
  public static function handle() {
    FormHandling::verifyPostNonce(self::ACTION);
    Auth::verifyDebugMode();
    Auth::verifyAdminCapability();

    $translator = LocaleTranslator::fromConfiguration();

    $log = InMemoryLog::empty();
    $response = AjaxResponse::from($log);

    $import = ImportSession::persisted(DataScope::none());
    StartImportSessionCommand::actionToggleToAjaxResponse($response, $import, $translator);

    if ($import === null) return $response->info($translator->noImportSessionFound());

    $import->flush();
    StartImportSessionCommand::actionToggleToAjaxResponse($response, $import, $translator);
    return $response->info($translator->importSessionFlushed());
  }

  /** @return Render */
  public static function ajaxButton(FlushImportSessionCommandTranslator $translator) {
    return ajaxButtonRender(
      self::label($translator),
      self::ACTION,
      ConfigurationFieldMode::adminDebug()->preventDisabling()
    );
  }
}
