<?php

namespace Wino;

interface SaveConfigurationCommandTranslator extends
ConfigurationShopIdTranslator,
ConfigurationPriceListTranslator,
FormHandlingTranslator {
  /** @return string */
  public function configurationUpdateNotPossibleDuringImport();
}

class SaveConfigurationCommand {
  public const ACTION = 'wino-save-configuration';

  /** @return void */
  public static function initialize() {
    addAjaxAction(self::ACTION, AjaxErrorHandler::notice(), function () {
      return self::handle();
    });
  }

  /** @return Render */
  public static function form(
    ?ImportSession $import,
    Render ...$content
  ) {
    return formRender(
      self::ACTION,
      Render::concat(...$content),
      FormRenderMode::create()
        ->singleUse()
        ->disable(ImportSession::importPreventConcurrency($import))
    );
  }

  /** @return AjaxResponse */
  public static function handle() {
    FormHandling::verifyPostNonce(self::ACTION);
    Auth::verifyAdminCapability();

    $context = WinoApiSynchronizationContext::fromConfiguration();
    $translator = LocaleTranslator::fromConfiguration();

    $apiClient = $context->getApiClient();

    $log = InMemoryLog::empty();
    $response = AjaxResponse::from($log);

    self::save($context, $translator);

    $response->dismissNotice(WinoApiClient::ERROR_CODE);
    $response->enableAction(self::ACTION, '');
    $response->replaceDomById(
      ConfigurationShopId::OPTION,
      ConfigurationShopId::renderInputWithSingleOptionAutosave($log, $apiClient, $translator)
    );
    $response->replaceDomById(
      ConfigurationPriceList::OPTION,
      ConfigurationPriceList::renderInput($log, $apiClient, $translator)
    );

    return $response;
  }

  /** @return void */
  private static function save(
    WinoApiSynchronizationContext $context,
    SaveConfigurationCommandTranslator $translator
  ) {
    FormHandling::verifyPluginVersion($translator);

    $import = ImportSession::persisted(DataScope::none());
    if (ImportSession::importPreventConcurrency($import)) throw msgException(
      $translator->configurationUpdateNotPossibleDuringImport()
    );

    if (isDebugModeEnabled()) ConfigurationApiUrl::save();

    ConfigurationApiKey::save();
    ConfigurationShopId::save();

    ConfigurationSkuBasedImportEnabled::save();
    ConfigurationCategoriesImportEnabled::save();
    ConfigurationRootCategory::save();
    ConfigurationExplicitCategoriesHierarchyEnabled::save();
    ConfigurationStockManagementOverrideEnabled::save();
    ConfigurationProductDescriptionFormat::save();

    ConfigurationProductsNamesUpdateEnabled::save();
    ConfigurationProductsDescriptionsUpdateEnabled::save();
    ConfigurationPricesUpdateEnabled::save();

    ConfigurationSynchronizationEnabled::save();
    ConfigurationMonitoringEnabled::save();

    WinoApiSynchronization::update($context);
  }
}
