<?php

namespace Wino;

interface StartImportSessionCommandTranslator extends
ConfigurationApiKeyTranslator,
ConfigurationShopIdTranslator,
ConfigurationPriceListTranslator,
ImportSessionTranslator {
  /** @return string */
  public function startImportSessionCommandLabel();
}

/**
 * Tax classes from legacy wino api taxes are intentionnaly not removed during import
 * to avoid potential missing data of old products. If an user want to delete them,
 * this can be done through the standard wordpress tax classes interface.
 */
class StartImportSessionCommand {
  public const ACTION = 'wino-start-import-session';

  /** @return string */
  public static function label(StartImportSessionCommandTranslator $translator) {
    return $translator->startImportSessionCommandLabel();
  }

  /** @return void */
  public static function initialize() {
    addAjaxAction(self::ACTION, AjaxErrorHandler::notice(), function () {
      return self::handle();
    });
  }

  /** @return AjaxResponse */
  public static function handle() {
    FormHandling::verifyPostNonce(self::ACTION);
    Auth::verifyAdminCapability();

    $translator = LocaleTranslator::fromConfiguration();

    ConfigurationApiKey::verify($translator);
    ConfigurationShopId::verify($translator);
    ConfigurationPriceList::verify($translator);
    ConfigurationRootCategory::verify($translator);

    $context = ImportContext::fromConfiguration();
    $import = ImportSession::startAttempt($context, $translator);

    $log = InMemoryLog::empty();
    $response = AjaxResponse::from($log);

    ConfigurationPriceList::save();

    self::actionToggleToAjaxResponse($response, $import, $translator);
    $response->callAction(CheckImportSessionProgressCommand::ACTION, AjaxErrorHandler::notice());

    return $response;
  }

  /** @return string */
  private static function importProgressLabel(
    ?ImportSession $import,
    StartImportSessionCommandTranslator $translator
  ) {
    $preventAction = ImportSession::importPreventConcurrency($import);
    $label = $import === null ? null : $import->progressLabel($translator);
    return $preventAction && $label !== null ? $label : self::label($translator);
  }

  /** @return void */
  public static function actionToggleToAjaxResponse(
    AjaxResponse $response,
    ?ImportSession $import,
    StartImportSessionCommandTranslator $translator
  ) {
    $preventAction = ImportSession::importPreventConcurrency($import);
    $label = self::importProgressLabel($import, $translator);

    $enableAction = !$preventAction;
    $response->toggleAction($enableAction, SaveConfigurationCommand::ACTION, $label);
    $response->toggleAction($enableAction, self::ACTION, $label);
  }

  /** @return Render */
  public static function ajaxButton(
    ?ImportSession $import,
    StartImportSessionCommandTranslator $translator
  ) {
    return ajaxButtonRender(
      self::importProgressLabel($import, $translator),
      self::ACTION,
      FormRenderMode::create()
        ->primary()
        ->singleUse()
        ->disable(ImportSession::importPreventConcurrency($import))
    );
  }
}
