<?php

namespace Wino;

interface CatalogAdminPageTranslator {
  /** @return string */
  public function importedProductsCountLabel(int $count);

  /** @return string */
  public function noImportedProducts();

  /** @return string */
  public function catalogLineSkuLabel();

  /** @return string */
  public function catalogLineStockLabel();

  /** @return string */
  public function catalogLineStatusLabel();

  /** @return string */
  public function catalogLineProductLabel();

  /** @return string */
  public function catalogLineVariantLabel();

  /** @return string */
  public function catalogLineWinoRefLabel();
}

class CatalogAdminPage {
  public const PARENT_MENU = 'edit.php?post_type=product';
  public const TITLE = 'Wino';
  public const MENU = 'Wino';
  public const MENU_SLUG = 'wino-catalog-page';

  /** @return void */
  public static function initialize() {
    add_action('admin_menu', function () {
      add_submenu_page(
        self::PARENT_MENU,
        self::TITLE,
        self::MENU,
        Auth::WOOCOMMERCE_ADMIN_CAPABILITY,
        self::MENU_SLUG,
        function () {
          self::render();
        }
      );
    });
  }

  /** @return string */
  public static function getUrl() {
    $menu = self::PARENT_MENU;
    $slug = self::MENU_SLUG;
    return admin_url("$menu&page=$slug");
  }

  /** @return void */
  public static function render() {
    Auth::verifyAdminCapability();
    Auth::verifyCurrentUserCapability(Auth::WOOCOMMERCE_ADMIN_CAPABILITY);

    $context = ImportContext::fromConfiguration();
    $translator = LocaleTranslator::fromConfiguration();

    $products = findAllMergedWinoProducts($context);

    $productsLines = [];
    foreach ($products as $product) {
      $skuLabel = $translator->catalogLineSkuLabel();
      $stockLabel = $translator->catalogLineStockLabel();
      $statusLabel = $translator->catalogLineStatusLabel();
      $productLabel = $translator->catalogLineProductLabel();
      $variantLabel = $translator->catalogLineVariantLabel();
      $winoRefLabel = $translator->catalogLineWinoRefLabel();

      $winoRef = $product->getWinoApiVariantId();
      $editionUrl = $product->getEditionPageUrl();
      $winoUrl = $product->getWinoPageUrl();
      $stock = $product->getStockQuantity();
      $sku = $product->getSku();

      $productLine = [
        $skuLabel => $sku === null ? '-' : $sku,
        $stockLabel => $product->getStockStatus() . "($stock)",
        $statusLabel => $product->getStatus(),
        $productLabel => renderHtmlLink($editionUrl, $product->getName()),
        $winoRefLabel => $winoRef === null ? '-' : renderHtmlLink($winoUrl, "$variantLabel $winoRef", "__blank"),
      ];
      $productsLines[] = $productLine;
    }
    $count = count($productsLines);
    $empty = $count === 0;
    $content = $empty
    ? renderHtmlLink(AdminPage::getUrl(), $translator->noImportedProducts())
    : htmlTable($productsLines, "text-align:center; width: 100%");

    $importedCountLabel = $translator->importedProductsCountLabel($count);
    echo renderAdminPageWrapStart($empty ? "" : " ($importedCountLabel)");
    echo $content;
    echo renderAdminPageWrapEnd();
    settings_errors(self::MENU_SLUG);
  }
}
