<?php

namespace Wino;

class ReleasePage {
  /** @return CustomRoute */
  public static function route() {
    $force = falsy();
    $enabled = $force || isDebugModeEnabled();

    return CustomRoute::from("download")->enable($enabled);
  }

  public const PLUGIN_ZIP_ENV_VAR_NAME = 'WINO_FOR_WOOCOMMERCE_PLUGIN_ZIP';

  /** @return string */
  public static function url() {
    return customRouteUrl(self::route());
  }

  /** @return string|null */
  public static function getPluginZipDownloadUrl(?string $zip = null) {
    $rootDir = dirname(__DIR__);

    $useZipMock = falsy();
    if (isDebugModeEnabled() && $useZipMock) return plugin_dir_url($rootDir . '/wino.php') . "wino.zip";

    $zipFileName = is_string($zip) ? $zip : getenv(self::PLUGIN_ZIP_ENV_VAR_NAME);
    if (!is_string($zipFileName)) return null;
    $trim = trim($zipFileName);
    if ($trim === '') return null;
    $pluginZipPath = $rootDir . '/' . $trim;
    if (!file_exists($pluginZipPath)) return null;
    $pluginFile = $rootDir . '/wino.php';
    $pluginZipUrl = plugin_dir_url($pluginFile) . $trim;
    return $pluginZipUrl;
  }

  /** @return void */
  public static function initialize() {
    registerCustomRoute(self::route());
  }

  /** @return string */
  private static function renderBody() {
    $url = self::getPluginZipDownloadUrl();
    if ($url === null) return "No zip available";
    return renderDownloadButton(basename($url), $url);
  }

  /** @return string */
  public static function render() {
    return renderHtmlPage(renderNoIndexHtmlPageHead(), self::renderBody());
  }
}
