<?php

namespace Wino;

interface ConfigurationApiKeyTranslator {
  /** @return string */
  public function configurationApiKeyLabel();

  /** @return string */
  public function missingRequiredApiKeyConfiguration();
}

/**
 * The API key is a bearer token provided to the user by the Wino team.
 */
class ConfigurationApiKey {
  public const OPTION = 'wino-api-key';
  public const DEFAULT = '';

  /** @return string */
  public static function label(ConfigurationApiKeyTranslator $translator) {
    return $translator->configurationApiKeyLabel();
  }

  /** @return string */
  public static function value() {
    return textConfigurationValue(self::OPTION, self::DEFAULT);
  }

  /** @return void */
  public static function save() {
    saveTextConfigurationValue(self::OPTION);
  }

  /** @return Render */
  public static function textField(ConfigurationApiKeyTranslator $translator) {
    return configurationTextFieldRender(
      self::label($translator),
      self::OPTION,
      self::value(),
      ConfigurationFieldMode::adminSave()
    );
  }

  /** @return bool */
  public static function validate(string $value) {
    return trim($value) !== "";
  }

  /** @return void */
  public static function verify(ConfigurationApiKeyTranslator $translator) {
    if (!self::validate(self::value())) throw msgException(
      $translator->missingRequiredApiKeyConfiguration()
    );
  }
}
