<?php

namespace Wino;

interface ConfigurationProductDescriptionFormatTranslator {
  /** @return string */
  public function configurationProductDescriptionFormatLabel();

  /** @return string */
  public function youCanInsertTheFollowingFieldsFromWino(string ...$fields);
}

class ConfigurationProductDescriptionFormat {
  public const DESCRIPTION_PLACEHOLDER = '{wino-description}';
  public const TASTING_NOTE_PLACEHOLDER = '{wino-note-de-degustation}';

  /** @return string */
  public static function defaultValue() {
    return str(
      self::DESCRIPTION_PLACEHOLDER,
      '<br><br>Note de dégustation : ',
      self::TASTING_NOTE_PLACEHOLDER
    );
  }

  /** @return string */
  public static function placeholderValue(WinoApiVariant $apiVariant, string $placeholder) {
    switch ($placeholder) {
      case self::DESCRIPTION_PLACEHOLDER:
      return $apiVariant->formattedDescription;

      case self::TASTING_NOTE_PLACEHOLDER:{
        $value = $apiVariant->tastingNote;
        return $value === null ? '' : $value;
      }

      default:
      return $placeholder;
    }
  }
  public const PLACEHOLDERS = [self::DESCRIPTION_PLACEHOLDER, self::TASTING_NOTE_PLACEHOLDER];

  public const OPTION = 'wino-product-description-format';

  /** @return string */
  public static function label(ConfigurationProductDescriptionFormatTranslator $translator) {
    return $translator->configurationProductDescriptionFormatLabel();
  }

  /** @return string */
  public static function notice(ConfigurationProductDescriptionFormatTranslator $translator) {
    return $translator->youCanInsertTheFollowingFieldsFromWino(
      self::DESCRIPTION_PLACEHOLDER,
      self::TASTING_NOTE_PLACEHOLDER
    );
  }

  /** @return string */
  public static function value() {
    return htmlConfigurationValue(self::OPTION, self::defaultValue());
  }

  /** @return void */
  public static function save() {
    saveHtmlConfigurationValue(self::OPTION);
  }

  /** @return Render */
  public static function textArea(ConfigurationProductDescriptionFormatTranslator $translator) {
    return configurationTextAreaRender(
      self::label($translator),
      self::OPTION,
      self::value(),
      ConfigurationFieldMode::adminSave(),
      self::notice($translator)
    );
  }
}
