<?php

namespace Wino;

class WinoCategoryUpdateProxy extends UpdateProxy {
  private \WP_Term $term;
  private DataScope $scope;
  private UpdateTermArgs $args;

  public function __construct(
    DataScope $scope,
    \WP_Term $term
  ) {
    parent::__construct();
    $this->scope = $scope;
    $this->term = $term;
    $this->args = new UpdateTermArgs();
  }

  /** @return self */
  public static function from(WinoCategory $category) {
    return new self($category->getScope(), $category->getTerm());
  }

  /** @return WinoCategory */
  public function getCategory() {
    return new WinoCategory($this->scope, $this->term);
  }

  /** @return \WP_Term */
  public function getTerm() {
    return $this->term;
  }

  /** @return void */
  private function setSlug(string $slug) {
    if ($this->skip($slug, $this->term->slug)) return;
    $this->args->data['slug'] = $slug;
  }

  /** @return void */
  private function setParent(int $parent) {
    if ($this->skip($parent, definedTermId($this->term->parent))) return;
    $this->args->data['parent'] = $parent;
  }


  /** @return void */
  private function setName(string $name) {
    if ($this->skip($name, $this->term->name)) return;
    $this->args->data['name'] = $name;
  }

  /** @return void */
  public function setArgs(UpdateTermArgs $args) {
    $slug = $args->getSlug();
    $parent = $args->getParent();
    $name = $args->getName();

    if ($slug !== null) $this->setSlug($slug);
    if ($parent !== null) $this->setParent($parent);
    if ($name !== null) $this->setName($name);
  }

  /**
   * @return UpdateTermArgs
   */
  public function termArgs() {
    return $this->args;
  }

  protected function applyCommit() {
    $this->term = updateWordpressTerm($this->term, $this->args);
    $this->args = new UpdateTermArgs();
  }
}
