<?php

namespace Wino;

/**
 * The DataScope main purpose is to allow creation
 * of test data while avoiding collision between tests.
 *
 * It is also used to prefix with "wino" the slugs, names
 * and meta-keys used by the plugin, in order to avoid
 * collision with non wino data.
 */
class DataScope {
  private const NO_SCOPE = "";
  private const WINO_ID_META_KEY = "winoID";
  private const WINO_API_ROOT_CATEGORY_ID = "00000001-0001-0001-0001-000000000001";
  private const WINO_API_ROOT_CATEGORY_NAME = "Wino";
  private const WINO_API_ROOT_CATEGORY_SLUG = "wino";
  private const WINO_PARENT_CATEGORY_ID_META_KEY = "winoParentCategoryID";
  private const WINO_TAX_ID_META_KEY = "winoTaxID";
  private const WINO_PRODUCT_ID_META_KEY = "winoProductID";
  private const WINO_CUSTOM_ROOT_CATEGORY_META_KEY = "winoCustomRootCategory";

  private string $prefix;
  public function __construct(string $prefix) {
    $this->prefix = $prefix;
  }

  /** @return string */
  public function getPrefix() {
    return $this->prefix;
  }

  /** @return string */
  public function scopedString(string $string) {
    $prefix = trim($this->getPrefix());
    return $prefix === self::NO_SCOPE ? $string : $prefix . '-' . $string;
  }

  /** @return string */
  public function scopedSlug(string $string) {
    return sanitize_title($this->scopedString($string));
  }

  /** @return self */
  public static function from(string $prefix) {
    return new self($prefix);
  }

  /** @return self */
  public static function none() {
    return new self(self::NO_SCOPE);
  }

  /** @return string */
  public function winoIdMetaKey() {
    return $this->scopedString(self::WINO_ID_META_KEY);
  }

  /** @return string */
  public function winoParentCategoryIdMetaKey() {
    return $this->scopedString(self::WINO_PARENT_CATEGORY_ID_META_KEY);
  }

  /** @return string */
  public function winoTaxIdMetaKey() {
    return $this->scopedString(self::WINO_TAX_ID_META_KEY);
  }

  /** @return string */
  public function winoProductIdMetaKey() {
    return $this->scopedString(self::WINO_PRODUCT_ID_META_KEY);
  }

  /** @return string */
  public function winoCustomRootCategoryMetaKey() {
    return $this->scopedString(self::WINO_CUSTOM_ROOT_CATEGORY_META_KEY);
  }

  /** @return string */
  public function winoApiRootCategoryId() {
    return $this->scopedString(self::WINO_API_ROOT_CATEGORY_ID);
  }

  /** @return string */
  public function winoApiRootCategoryName() {
    return $this->scopedString(self::WINO_API_ROOT_CATEGORY_NAME);
  }

  /** @return string */
  public function winoApiRootCategorySlug() {
    return $this->scopedSlug(self::WINO_API_ROOT_CATEGORY_SLUG);
  }

  /** @return WinoApiCategory */
  public function winoApiRootCategory(?string $shopId) {
    return new WinoApiCategory(
      new WinoApiShopIdentity($shopId === null ? "" : $shopId),
      $this->winoApiRootCategoryId(),
      null,
      $this->winoApiRootCategoryName()
    );
  }

  /** @return bool */
  private static function isWinoIdMetaKeyFromAnyScope(string $key) {
    return strEndsWith($key, self::WINO_ID_META_KEY);
  }

  /** @return string[] */
  public static function findDistinctScopeWinoIdMetaKeys() {
    $metaKeys = [];
    $allMetaKeys = findDistinctWordpressTermMetaKeys();
    foreach ($allMetaKeys as $key) if (self::isWinoIdMetaKeyFromAnyScope($key)) $metaKeys[] = $key;
    return $metaKeys;
  }

  /** @return bool */
  public static function termIdIsFromAnyScope(?int $termId) {
    if ($termId === null) return false;
    $term = findWordpressTermById($termId);
    return $term === null ? false : self::termIsFromAnyScope($term);
  }

  /** @return bool */
  public static function termIsFromAnyScope(\WP_Term $term) {
    $metaKeys = findWordpressTermMetaKeys($term->term_id);
    foreach ($metaKeys as $key) if (self::isWinoIdMetaKeyFromAnyScope($key)) return true;
    $parent = findWordpressTermById($term->parent);
    if ($parent === null) return false;
    return self::termIsFromAnyScope($parent);
  }
}
