<?php

namespace Wino;

/** @return string */
function monitoringDirPath() {
  $pluginDir = winoPluginDir();
  return "$pluginDir/monitoring";
}

/** @return string */
function monitoringLogFilePrefix() {
  return 'monitoring.wino.';
}

/** @return string */
function monitoringLogFileSuffix() {
  return '.log';
}

/** @return string */
function monitoringLogFileBasename(?string $formattedDate = null) {
  $prefix = monitoringLogFilePrefix();
  $dateStr = $formattedDate === null ? date_format(dateTimeNow(), "Y-m-d") : $formattedDate;
  $suffix = monitoringLogFileSuffix();
  return "$prefix$dateStr$suffix";
}


/** @return string */
function monitoringLogFilePath(?string $formattedDate = null) {
  $pluginDir = monitoringDirPath();
  $basename = monitoringLogFileBasename($formattedDate);
  return "$pluginDir/$basename";
}

/** @return string | null */
function monitoringLogFileDateFromPath(string $path) {
  $prefix = monitoringLogFilePrefix();
  $suffix = monitoringLogFileSuffix();
  $basename = basename($path);
  if (!strStartsWith($basename, $prefix)) return null;
  if (!strEndsWith($basename, $suffix)) return null;
  $trimPrefix = substr($basename, strlen($prefix));
  return substr($trimPrefix, 0, strlen($trimPrefix) - strlen($suffix));
}


/** @return string[] */
function monitoringDirFiles() {
  $files = [];
  $dir = monitoringDirPath();
  if (!file_exists($dir)) return $files;
  $scan = scandir($dir);
  foreach ($scan as $file)
  if (strStartsWith($file, 'monitoring.wino.'))
  if (strEndsWith($file, '.log'))
  if (is_file(path_join($dir, $file))) $files[] = $file;

  return $files;
}

class MonitoringContext {
  public bool $monitoringEnabled;

  private function __construct(bool $monitoringEnabled) {
    $this->monitoringEnabled = $monitoringEnabled;
  }

  /** @return self */
  public static function fromConfiguration() {
    return new self(
      ConfigurationMonitoringEnabled::value()
    );
  }

  /** @return self */
  public static function fromConfigurationDefaults() {
    return new self(
      ConfigurationMonitoringEnabled::DEFAULT
    );
  }

  /** @return self */
  public function emptyCacheCopy() {
    return new self(
      $this->monitoringEnabled
    );
  }

  /**
   * @param mixed $log
   * @param mixed[] $data
   * @return void
   */
  public function log($log, ...$data) {
    if (!$this->monitoringEnabled) return;
    $string = prettyLogString($log, ...$data);
    $now = dateTimeNow();
    $datetime = formatDateTime($now);
    $line = "[$datetime] $string\n";
    $filePath = monitoringLogFilePath();
    $dir = dirname($filePath);
    if (!file_exists($dir)) mkdir($dir, 0777, true);
    try {
      file_put_contents($filePath, $line, FILE_APPEND | LOCK_EX);
    } catch (\Throwable $err) {
      prettyLog($err);
    } finally {
      writeLog("WINO MONITORING - $string");
    }
  }
}
