<?php

namespace Wino;

abstract class UpdateProxy {
  protected int $updateCount;

  protected function __construct() {
    $this->updateCount = 0;
  }

  /**
   * @template T
   * @param T $newValue
   * @param T $oldValue
   * @return bool
   */
  protected function skip($newValue, $oldValue) {
    $change = $newValue !== $oldValue;
    $this->updateCount += $change ? 1 : 0;
    return !$change;
  }

  /** @return bool */
  public function hasUpdate() {
    return $this->updateCount > 0;
  }

  /** @return void */
  protected function prepareCommit() {
  }

  /** @return void */
  abstract protected function applyCommit();

  /** @return bool */
  public function commit() {
    $this->prepareCommit();
    if (!$this->hasUpdate()) return false;
    $this->applyCommit();
    $this->updateCount = 0;
    return true;
  }
}
