<?php

namespace Wino;

/** @return string[] */
function getWcTaxClassSlugs() {
  return assertedStringsArray(\WC_Tax::get_tax_class_slugs());
}

/** @return object[] */
function getWcRatesForTaxClass(string $taxClassSlug) {
  return assertedObjectsArray(\WC_Tax::get_rates_for_tax_class($taxClassSlug));
}

/** @return WinoTaxClass|null */
function findWinoTaxClassBySlug(ImportContext $context, string $slug) {
  $scope = $context->getScope();
  if (!WinoTaxKeys::isWinoTaxClassSlug($scope, $slug)) return null;
  $cached = $context->getCachedTaxClass($slug);
  if ($cached !== null) return $cached;

  $data = \WC_Tax::get_tax_class_by("slug", $slug);
  if (!is_array($data)) return null;
  return $context->cacheTaxClass(WinoTaxClass::from($scope, $data));
}

/**
 * @return WinoTaxClass[]
 */
function findAllWinoTaxClasses(ImportContext $context) {
  $scope = $context->getScope();
  $slugs = getWcTaxClassSlugs();
  $winoClasses = [];
  foreach ($slugs as $slug) {
    if (!WinoTaxKeys::isWinoTaxClassSlug($scope, $slug)) continue;
    $taxClass = findWinoTaxClassBySlug($context, $slug);
    if ($taxClass === null) throw exception("Missing tax class with slug $slug");
    $winoClasses[] = $taxClass;
  }
  return $winoClasses;
}

/** @return WinoTaxRate[] */
function findWinoTaxClassRates(
  ImportContext $context,
  WinoTaxClass $taxClass
) {
  $rates = getWcRatesForTaxClass($taxClass->slug);
  $taxRates = [];
  foreach ($rates as $rate) $taxRates[] = $context->cacheTaxRate(
    WinoTaxRate::from($taxClass->getScope(), objectAsArray($rate))
  );
  return $taxRates;
}

/** @return WinoTaxRate|null */
function findWinoTaxClassRateByName(
  ImportContext $context,
  WinoTaxClass $taxClass,
  string $rateName
) {
  $cached = $context->getCachedTaxRate($rateName);
  if ($cached !== null) return $cached->taxClassSlug === $taxClass->slug ? $cached : null;

  $rates = findWinoTaxClassRates($context, $taxClass);
  foreach ($rates as $rate) if ($rate->name === $rateName) return $rate;
  return null;
}

/**
 * @return WinoTaxRate[]
 */
function findAllWinoTaxRates(ImportContext $context) {
  $taxClasses = findAllWinoTaxClasses($context);
  $taxRates = [];
  foreach ($taxClasses as $taxClass) {
    $rates = findWinoTaxClassRates($context, $taxClass);
    foreach ($rates as $rate) $taxRates[] = $rate;
  }
  return $taxRates;
}
