<?php

namespace Wino;

/**
 * @template K of array-key
 * @template T
 * @param array<K, T> $array
 * @param K $key
 * @return T|null
 */
function arrayElement(array $array, $key) {
  return key_exists($key, $array) ? $array[$key] : null;
}

/**
 * @template T
 * @param T[] $list
 * @return T|null
 */
function listElement(array $list, int $index) {
  return arrayElement($list, $index);
}

/**
 * @template T
 * @param T[] $array
 * @param Closure(T, T): int $compareFunction
 *
 * @return T[]
 */
function sortArrayCopy(array $array, callable $compareFunction) {
  $copy = [];
  foreach ($array as $item) $copy[] = $item;
  usort($copy, $compareFunction);
  return $copy;
}

/**
 * @template T
 * @param T[] $array
 * @param Closure(T): int $getInt
 *
 * @return T[]
 */
function sortArrayCopyByInt(array $array, callable $getInt) {
  return sortArrayCopy(
    $array,
    /**
     * @param T $a
     * @param T $b
     */
    function ($a, $b) use ($getInt) {
      $intA = call_user_func($getInt, $a);
      $intB = call_user_func($getInt, $b);
      return $intA - $intB;
    }
  );
}

/**
 * @template T
 * @param T[] $array
 * @param Closure(T): \DateTime $getDate
 *
 * @return T[]
 */
function sortArrayCopyByDate(array $array, callable $getDate) {
  return sortArrayCopyByInt(
    $array,
    /** @param T $value */
    function ($value) use ($getDate) {
      $date = call_user_func($getDate, $value);
      return assertedInt($date->getTimestamp());
    }
  );
}

/**
 * @template T
 * @param T[] $array
 * @param Closure(T): string $getString
 *
 * @return T[]
 */
function sortArrayCopyByString(array $array, callable $getString) {
  return sortArrayCopy(
    $array,
    /**
     * @param T $a
     * @param T $b
     */
    function ($a, $b) use ($getString) {
      $strA = call_user_func($getString, $a);
      $strB = call_user_func($getString, $b);
      return strcmp($strA, $strB);
    }
  );
}
