<?php

namespace Wino;

class PaginatedDataIteratorState {
  public bool $hasNextPage;
  public ?array $currentPageItems;
  public int $currentPageIndex;

  public function __construct(
    ?array $currentPageItems,
    ?int $currentPageIndex,
    bool $hasNextPage
  ) {
    $this->currentPageItems = $currentPageItems;
    $this->currentPageIndex = $currentPageIndex ?? 0;
    $this->hasNextPage = $hasNextPage;
  }
}

/** @template T */
abstract class PaginatedDataIterator {
  protected PaginatedDataIteratorState $state;

  /** @var T|null */
  protected $currentItem;

  /** @return PaginatedDataIteratorState */
  public function getCurrentState() {
    return $this->state;
  }

  /** @return void */
  protected function resumeState(PaginatedDataIteratorState $state) {
    $this->state = $state;
    $currentPageItems = $state->currentPageItems;
    $currentPageIndex = $state->currentPageIndex;
    /** @var mixed */
    $currentItem = listElement($currentPageItems ?? [], $currentPageIndex);
    /** @psalm-suppress MixedAssignment */
    $this->currentItem = $currentItem;
  }

  /** @return int */
  public function getCurrentPageIndex() {
    return $this->state->currentPageIndex;
  }

  public function __construct() {
    $state = new PaginatedDataIteratorState(null, null, false);
    $this->state = $state;
    $this->resumeState($state);
  }

  /** @return T[] */
  abstract protected function nextPageItems();

  /** @return bool */
  public function next() {
    $state = $this->state;
    $items = $state->currentPageItems;
    $index = $state->currentPageIndex;
    if ($items === null) {
      $items = $this->nextPageItems();
      $index = 0;
      $state->currentPageItems = $items;
      $state->currentPageIndex = $index;
    }
    $count = count($items);
    if ($index < $count) {
      /** @var mixed */
      $currentItem = $items[$index];
      $state->currentPageIndex = $index + 1;
      /** @psalm-suppress MixedAssignment */
      $this->currentItem = $currentItem;
      return $currentItem !== null;
    }
    if (!$state->hasNextPage) return false;
    $state->currentPageItems = null;
    return $this->next();
  }

  /** @return T */
  protected function currentPageItem() {
    $item = $this->currentItem;
    if ($item !== null) return $item;
    throw error("Null item found.");
  }
}
