<?php

namespace Wino;

class WinoTranslatorFr extends FrFormHandlingTranslator implements WinoTranslator {
  public function debugFieldsGroupLabel() {
    $wpDebug = Constant::WORDPRESS_DEBUG;
    $winoDebug = Constant::WINO_DEBUG;
    return msg(
      "Les champs suivants sont uniquement accessibles en mode $wpDebug & $winoDebug.",
      "Ne les utilisez pas sans savoir ce que vous faites."
    );
  }

  public function startImportSessionCommandLabel() {
    $config = strtolower(self::configurationPriceListLabel());
    return "Choisir une $config et démarrer l'import";
  }

  /** @return string */
  public function configurationPriceListLabel() {
    return "Grille tarifaire";
  }

  /** @return string */
  public function importRequiredToConfirmAPriceListUpdate() {
    return "Import requis pour valider un changement de grille tarifaire";
  }

  /** @return string */
  public function noPriceListSelectedOptionLabel() {
    return "Pas de grille tarifaire sélectionnée";
  }

  /** @return string */
  public function categoriesImportEnabledLabel() {
    return "Activer l'import des catégories Wino";
  }

  /** @return string */
  public function explicitCategoriesHierarchyEnabledLabel() {
    return "Cocher aussi les catégories parentes";
  }

  /** @return string */
  public function nothingToImport() {
    return "Rien à importer.";
  }

  /** @return string */
  public function anImportIsRunningYet() {
    return "Un import est déjà en cours.";
  }

  /** @return string */
  private static function importProgressCount(string $label, int $count) {
    return $count === 0 ? "$label..." : "$label ($count)";
  }

  /** @return string */
  public function categoriesImportInProgress(int $importedCategories) {
    return self::importProgressCount("Import des catégories en cours", $importedCategories);
  }

  /** @return string */
  public function taxesImportInProgress(int $importedTaxes) {
    return self::importProgressCount("Import des taxes en cours", $importedTaxes);
  }

  /** @return string */
  public function productsImportInProgress(int $importedProducts) {
    return self::importProgressCount("Import des produits en cours", $importedProducts);
  }

  /** @return string */
  public function unusedCategoriesDeletionInProgress() {
    return "Suppression des catégories Wino non utilisées en cours...";
  }

  /** @return string */
  public function flushImportSessionCommandLabel() {
    return "Supprimer la session d'import";
  }

  /** @return string */
  public function importSessionFlushed() {
    return "La session d'import a été supprimée.";
  }

  /** @return string */
  public function noImportSessionFound() {
    return "Aucune session d'import n'a été trouvée.";
  }

  /** @return string */
  public function configurationStockManagementOverrideEnabledLabel() {
    return "Activer la gestion des stocks sur les produits importés";
  }

  /** @return string */
  public function nothingToDeleteLabel() {
    return 'Rien à supprimer.';
  }

  /** @return string */
  public function importReportTypeLabel() {
    return 'Type';
  }

  /** @return string */
  public function importReportNameLabel() {
    return 'Nom';
  }

  /** @return string */
  public function importReportWordpressRefLabel() {
    return 'Wordpress Ref.';
  }

  /** @return string */
  public function importReportWinoRefLabel() {
    return 'Wino Ref.';
  }

  /** @return string */
  public function updateFieldsGroupLabel() {
    return "Mises à jour avec les données en provenance de Wino";
  }

  /** @return string */
  public function adminPageTitle() {
    return 'Plugin Wino pour WooCommerce';
  }

  /** @return string */
  public function importedProductsCountLabel(int $count) {
    return "$count produits importés";
  }

  /** @return string */
  public function noImportedProducts() {
    return "Aucun produit importé";
  }

  /** @return string */
  public function catalogLineSkuLabel() {
    return "SKU";
  }

  /** @return string */
  public function catalogLineStockLabel() {
    return "Stock";
  }

  /** @return string */
  public function catalogLineStatusLabel() {
    return "Status";
  }

  /** @return string */
  public function catalogLineProductLabel() {
    return "Produit";
  }

  /** @return string */
  public function catalogLineVariantLabel() {
    return "Variante";
  }

  /** @return string */
  public function catalogLineWinoRefLabel() {
    return self::importReportWinoRefLabel();
  }

  /** @return string */
  private static function importReportKind(string $kind) {
    switch ($kind) {
      case CategoryImportResult::WINO_REF_KIND:
      return "Catégorie";

      case CategoryImportResult::WORDPRESS_REF_KIND:
      return "Catégorie produit";

      case TaxClassImportResult::WINO_REF_KIND:
      return "Taxe";

      case ProductImportResult::WORDPRESS_REF_KIND:
      return "Produit";

      case ProductImportResult::WINO_REF_KIND:
      return "Variante";

      default:
      return $kind;
    }
  }

  /** @return string */
  private static function importReportEventTargetGenderSuffix(string $eventTargetKind) {
    switch ($eventTargetKind) {
      case CategoryImportResult::WINO_REF_KIND:
      case TaxClassImportResult::WINO_REF_KIND:
      case ProductImportResult::WINO_REF_KIND:
      return "e";

      default:
      return "";
    }
  }

  /** @return string */
  private static function importReportEventKind(string $eventKind, string $eventTargetKind) {
    $targetGenderSuffix = self::importReportEventTargetGenderSuffix($eventTargetKind);
    switch ($eventKind) {
      case DataImportEvent::CREATED:
      return "créé$targetGenderSuffix";

      case DataImportEvent::UPDATED:
      return "mis$targetGenderSuffix à jour";

      case DataImportEvent::ARCHIVED:
      return "archivé$targetGenderSuffix";

      case DataImportEvent::DELETED:
      return "supprimé$targetGenderSuffix";

      default:
      return $eventKind;
    }
  }

  /** @return string */
  public function importReportType(string $eventKind, string $eventTargetKind) {
    $target = self::importReportKind($eventTargetKind);
    $event = self::importReportEventKind($eventKind, $eventTargetKind);
    return "$target $event";
  }

  /** @return string */
  public function importReportWordpressRef(string $refKind, string $ref) {
    $kind = self::importReportKind($refKind);
    return "$kind $ref";
  }

  /** @return string */
  public function importReportWinoRef(string $refKind, string $ref) {
    $kind = self::importReportKind($refKind);
    return "$kind $ref";
  }

  /** @return string */
  public function deleteDataCommandLabel() {
    return "Supprimer les données en provenance de Wino";
  }

  /** @return string */
  public function deleteDataCommandConfirmationMessage() {
    return "Êtes-vous sûr de vouloir supprimer les données Wino ?";
  }

  /** @return string */
  public function configurationUpdateNotPossibleDuringImport() {
    return "Impossible de mettre à jour la configuration: Un import est en cours...";
  }

  /** @return string */
  public function invalidApiUrl() {
    return msg("Option", jsonString($this->configurationApiUrlLabel()), "non valide.");
  }

  /** @return string */
  public function impossibleToReachApiUrl(string $url) {
    $debugMode = isDebugModeEnabled();
    $debugMessageEnd = msg(
      ", veuillez vérifier l'option",
      jsonString($this->configurationApiUrlLabel())
    );

    return str(
      "Impossible de joindre l'API Wino à l'adresse ",
      jsonString($url),
      $debugMode ? $debugMessageEnd : '',
      '.'
    );
  }

  /** @return string */
  public function connectionRefusedToApiUrl(string $url) {
    return str(
      "Connection à l'API Wino (",
      jsonString($url),
      ") refusée, veuillez vérifier l'option ",
      jsonString($this->configurationApiKeyLabel())
    );
  }

  /** @return string */
  public function configurationApiKeyLabel() {
    return "Wino API Key";
  }

  /** @return string */
  private static function missingRequiredConfiguration(string $label) {
    return msg("Configuration requise :", jsonString($label));
  }

  /** @return string */
  public function missingRequiredApiKeyConfiguration() {
    return self::missingRequiredConfiguration($this->configurationApiKeyLabel());
  }

  /** @return string */
  public function configurationShopIdLabel() {
    return "Boutique";
  }

  /** @return string */
  public function missingRequiredShopIdConfiguration() {
    return self::missingRequiredConfiguration($this->configurationShopIdLabel());
  }

  /** @return string */
  public function apiKeyIsRequiredToSelectShop() {
    return msg(
      "L'option",
      jsonString($this->configurationApiKeyLabel()),
      "est requise pour choisir la boutique."
    );
  }

  /** @return string */
  public function missingRequiredPriceListConfiguration() {
    return self::missingRequiredConfiguration($this->configurationPriceListLabel());
  }

  /** @return string */
  public function apiKeyAndShopIdAreRequiredToSelectPriceList() {
    return msg(
      "Les options",
      jsonString($this->configurationApiKeyLabel()),
      "et",
      jsonString($this->configurationShopIdLabel()),
      "sont requises pour choisir une grille tarifaire."
    );
  }

  /** @return string */
  public function configurationApiUrlLabel() {
    return "URL de l'API Wino";
  }

  /** @return string */
  public function configurationSkuBasedImportEnabledLabel() {
    return "Activer l'import par SKU";
  }

  /** @return string */
  public function configurationProductDescriptionFormatLabel() {
    return "Description des produits";
  }

  /** @return string */
  public function youCanInsertTheFollowingFieldsFromWino(string ...$fields) {
    return msg(
      "Vous pouvez insérer les champs suivants en provenance de Wino :",
      ...$fields
    );
  }

  /** @return string */
  public function configurationProductsNamesUpdateEnabledLabel() {
    return "Mettre à jour le nom des produits";
  }

  /** @return string */
  public function configurationProductsDescriptionsUpdateEnabledLabel() {
    return "Mettre à jour la description des produits";
  }

  /** @return string */
  public function configurationPricesUpdateEnabledLabel() {
    return "Mettre à jour les prix de vente";
  }

  /** @return string */
  public function configurationSynchronizationEnabledLabel() {
    return "Activer la synchronisation entre Wino et WooCommerce";
  }

  /** @return string */
  public function configurationMonitoringEnabledLabel() {
    return "Activer le monitoring Wino";
  }

  /** @return string */
  public function configurationLocaleLabel() {
    return "Langue";
  }

  /** @return string */
  public function showAllImportedProductsOnDedicatedPage() {
    return renderHtmlLink(
      CatalogAdminPage::getUrl(),
      "Voir les produits importés"
    );
  }

  /** @return string */
  public function theFollowingOperationsWereSuccessful() {
    return "Les opérations d'import suivantes ont été effectuées";
  }

  /** @return string */
  public function rootCategoryLabel() {
    return "Choisir une catégorie racine";
  }

  /** @return string */
  public function noRootCategoryOptionLabel() {
    return "Aucune";
  }

  /** @return string */
  public function defaultWinoRootCategoryOptionLabel() {
    return "Wino";
  }

  /** @return string */
  public function apiRootCategoryImportErrorMessage(WinoApiCategory $apiRootCategory) {
    $label = self::defaultWinoRootCategoryOptionLabel();
    $root = jsonString($apiRootCategory);
    return "Impossible d'importer la catégorie racine $label: $root";
  }

  /** @return string */
  public function invalidRootCategoryErrorMessage(\WP_Term $root) {
    $id = $root->term_id;
    $name = $root->name;
    return "La catégorie \"$name\" (id: \"$id\") n'est pas une catégorie racine valide.";
  }

  /**
   * @param \WP_Term[] $duplicateTerms
   * @return string
   */
  public function duplicateCategoryErrorMessage(string $winoApiId, $duplicateTerms) {
    $duplicates = jsonString($duplicateTerms);
    return "Doublons détectés parmi les catégories Wino (id: $winoApiId) => $duplicates";
  }

  /** @return string */
  private static function duplicateSkuRef(
    string $name,
    ?int $postId,
    ?string $winoId,
    string $skuLabel,
    ?string $sku
  ) {
    $refName = jsonString($name);
    $refs = [];
    if ($postId !== null) $refs[] = "id: $postId";
    if ($winoId !== null) $refs[] = "winoId: $winoId";
    if ($sku !== null) $refs[] = "$skuLabel: $sku";
    $refsJoins = join(", ", $refs);
    return "$refName ($refsJoins)";
  }

  /** @return string */
  private static function duplicateProductRef(
    WinoProduct $product
  ) {
    return self::duplicateSkuRef(
      $product->getName(),
      $product->getPostId(),
      $product->getWinoApiVariantId(),
      "ugs",
      $product->getSku()
    );
  }

  /** @return string */
  public function duplicateSkuErrorMessage(
    WinoApiVariant $importedApiVariant,
    WinoProduct $productWithSameSku,
    ?WinoProduct $productWithSameVariantId
  ) {
    $importedVariantRef = self::duplicateSkuRef(
      $importedApiVariant->formattedName,
      null,
      $importedApiVariant->id,
      "sku",
      $importedApiVariant->stockKeepingUnit
    );
    $productWithSameSkuRef = self::duplicateProductRef($productWithSameSku);
    $productWithSameVariantIdRef = $productWithSameVariantId === null ? null : self::duplicateProductRef(
      $productWithSameVariantId
    );

    return msg(
      "UGS/SKU dupliquée détectée pendant l'import de $importedVariantRef.",
      $productWithSameVariantIdRef === null
      ? ""
      : "Un produit correspondant existe déjà : $productWithSameVariantIdRef.",
      "Un produit avec le même UGS existe déjà : $productWithSameSkuRef."
    );
  }
}
