<?php

namespace Wino;

class WinoApiCategoryParent {
  public string $id;

  public function __construct(string $id) {
    $this->id = $id;
  }

  public const FIELDS_QUERY = "id";

  /** @return self */
  public static function from(array $data) {
    return new self(
      nonBlankStringData($data, "id")
    );
  }
}

/** @extends GraphQLPaginatedQueryEdgeNodeIterator<WinoApiCategory> */
class WinoApiCategoryQueryAllIterator extends GraphQLPaginatedQueryEdgeNodeIterator {
  /** @return WinoApiCategory */
  public function currentObject() {
    return WinoApiCategory::from($this->currentNode());
  }
}

class WinoApiCategory {
  public WinoApiShopIdentity $shop;
  public string $id;
  public ?\DateTime $archivedAt;
  public string $name;
  public ?WinoApiCategoryParent $parent;

  public function __construct(
    WinoApiShopIdentity $shop,
    string $id,
    ?\DateTime $archivedAt,
    string $name,
    ?WinoApiCategoryParent $parent = null
  ) {
    $this->id = $id;
    $this->archivedAt = $archivedAt;
    $this->name = $name;
    $this->parent = $parent;
    $this->shop = $shop;
  }

  /** @return string */
  public static function fieldsQuery() {
    return WinoApiClient::graphQLFieldsQuery(
      'id',
      'archivedAt',
      'name',
      WinoApiClient::graphQLNestedFieldsQuery('shop', WinoApiShopIdentity::FIELDS_QUERY),
      WinoApiClient::graphQLNestedFieldsQuery('parent', WinoApiCategoryParent::FIELDS_QUERY),
    );
  }

  /** @return self */
  public static function from(array $data) {
    $parent = optionalArrayData($data, 'parent');
    return new self(
      WinoApiShopIdentity::from(arrayData($data, 'shop')),
      nonBlankStringData($data, "id"),
      optionalDateData($data, "archivedAt", TimeUnit::javascriptTimestampLike()),
      nonBlankStringData($data, "name"),
      $parent === null ? null : WinoApiCategoryParent::from($parent),
    );
  }

  public const NODE_QUERY_NAME = "Category";
  public const NODE_QUERY_RESOLVER = "category";

  /** @return WinoApiCategory */
  public static function queryById(WinoApiClient $api, string $id) {
    return self::from($api->postGraphQLNodeByIdQuery(
      self::NODE_QUERY_NAME,
      "",
      self::NODE_QUERY_RESOLVER,
      "",
      $id,
      self::fieldsQuery(),
      []
    ));
  }

  public const COLLECTION_QUERY_NAME = "Categories";
  public const COLLECTION_QUERY_RESOLVER = "categories";

  /**
   * @param array<string, mixed> $queryVars
   *
   * @return WinoApiCategoryQueryAllIterator
   */
  public static function queryAllIterator(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop,
    string $queryName = self::COLLECTION_QUERY_NAME,
    string $queryArgs = "",
    string $filterByArgs = "",
    string $resolverArgs = "",
    array $queryVars = []
  ) {
    return new WinoApiCategoryQueryAllIterator(
      $winoApi,
      GraphQLPaginatedQueryRoot::none(),
      $queryName,
      WinoApiShopIdentity::queryArgs($shop, $queryArgs),
      self::COLLECTION_QUERY_RESOLVER,
      WinoApiShopIdentity::resolverArgs($shop, $filterByArgs, $resolverArgs),
      self::fieldsQuery(),
      WinoApiShopIdentity::queryVars($shop, $queryVars)
    );
  }

  /**
   * Using `queryAllIterator` might be prefered in order to reduce
   * memory footprint. This is especially true if you expect lot of
   * items in the array which could lead to reach the php memory limit.
   *
   * @return WinoApiCategory[]
   */
  public static function queryAll(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop
  ) {
    return self::queryAllIterator($winoApi, $shop)->queryAll();
  }

  /** @return null|string */
  public function getParentId() {
    $parent = $this->parent;
    return $parent === null ? null : $parent->id;
  }

  /** @return bool */
  public function isArchived() {
    return $this->archivedAt !== null;
  }

  /** @return WinoApiCategoryParent */
  public function asParent() {
    return new WinoApiCategoryParent($this->id);
  }
}
