<?php

namespace Wino;

/** @extends GraphQLPaginatedQueryEdgeNodeIterator<WinoApiPrice> */
class WinoApiPriceQueryAllIterator extends GraphQLPaginatedQueryEdgeNodeIterator {
  /** @return WinoApiPrice */
  public function currentObject() {
    return WinoApiPrice::from($this->currentNode());
  }
}

class WinoApiPrice {
  public WinoApiShopIdentity $shop;
  public string $id;
  public ?\DateTime $archivedAt;
  public string $name;

  public function __construct(
    WinoApiShopIdentity $shop,
    string $id,
    ?\DateTime $archivedAt,
    string $name
  ) {
    $this->id = $id;
    $this->shop = $shop;
    $this->archivedAt = $archivedAt;
    $this->name = $name;
  }

  public const FIELDS_QUERY = "id, name, archivedAt, shop { " . WinoApiShopIdentity::FIELDS_QUERY . " }";

  /** @return self */
  public static function from(array $data) {
    return new self(
      WinoApiShopIdentity::from(arrayData($data, "shop")),
      nonBlankStringData($data, 'id'),
      optionalDateData($data, "archivedAt", TimeUnit::javascriptTimestampLike()),
      nonBlankStringData($data, 'name')
    );
  }

  public const NODE_QUERY_NAME = "Price";
  public const NODE_QUERY_RESOLVER = "price";

  public const COLLECTION_QUERY_NAME = "Prices";
  public const COLLECTION_QUERY_RESOLVER = "prices";

  /**
   * @param array<string, mixed> $queryVars
   * @return WinoApiPriceQueryAllIterator
   */
  public static function queryAllIterator(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop,
    string $queryName = self::COLLECTION_QUERY_NAME,
    string $queryArgs = "",
    string $filterByArgs = "",
    string $resolverArgs = "",
    array $queryVars = []
  ) {
    return new WinoApiPriceQueryAllIterator(
      $winoApi,
      GraphQLPaginatedQueryRoot::none(),
      $queryName,
      WinoApiShopIdentity::queryArgs($shop, $queryArgs),
      self::COLLECTION_QUERY_RESOLVER,
      WinoApiShopIdentity::resolverArgs($shop, $filterByArgs, $resolverArgs),
      self::FIELDS_QUERY,
      WinoApiShopIdentity::queryVars($shop, $queryVars)
    );
  }

  /**
   * Using `queryAllIterator` might be prefered in order to reduce
   * memory footprint. This is especially true if you expect lot of
   * items in the array which could lead to reach the php memory limit.
   *
   * @return WinoApiPrice[]
   */
  public static function queryAll(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop
  ) {
    return self::queryAllIterator($winoApi, $shop)->queryAll();
  }

  /** @return bool */
  public function isArchived() {
    return $this->archivedAt !== null;
  }
}
