<?php

namespace Wino;

/** @extends GraphQLPaginatedQueryEdgeNodeIterator<WinoApiStockActivity> */
class WinoApiStockActivityQueryAllIterator extends GraphQLPaginatedQueryEdgeNodeIterator {
  /** @return WinoApiStockActivity */
  public function currentObject() {
    return WinoApiStockActivity::from($this->currentNode());
  }
}

class WinoApiStockActivity {
  public string $id;
  public const ID_KEY = 'id';
  public \DateTime $createdAt;
  public const CREATED_AT_KEY = 'createdAt';
  public WinoApiShopIdentity $shop;
  public const SHOP_KEY = 'shop';
  public ?WinoApiVariantIdentity $variant;
  public const VARIANT_KEY = 'variant';
  public string $kind;
  public const KIND_KEY = 'kind';
  public int $quantity;
  public const QUANTITY_KEY = 'quantity';
  public ?string $comment;
  public const COMMENT_KEY = 'comment';

  public const SALE_KIND = "SALE";
  public function __construct(
    string $id,
    \DateTime $createdAt,
    WinoApiShopIdentity $shop,
    ?WinoApiVariantIdentity $variant,
    string $kind,
    int $quantity,
    ?string $comment
  ) {
    $this->id = $id;
    $this->createdAt = $createdAt;
    $this->shop = $shop;
    $this->variant = $variant;
    $this->kind = $kind;
    $this->quantity = $quantity;
    $this->comment = $comment;
  }

  /** @return string */
  public static function fieldsQuery(bool $queryFullVariant) {
    return WinoApiClient::graphQLFieldsQuery(
      self::ID_KEY,
      self::CREATED_AT_KEY,
      self::KIND_KEY,
      self::QUANTITY_KEY,
      self::COMMENT_KEY,
      WinoApiClient::graphQLNestedFieldsQuery(
        self::SHOP_KEY,
        WinoApiShopIdentity::FIELDS_QUERY
      ),
      WinoApiClient::graphQLNestedFieldsQuery(
        self::VARIANT_KEY,
        $queryFullVariant ? WinoApiVariant::fieldsQuery() : WinoApiVariantIdentity::fieldsQuery()
      )
    );
  }

  /** @return self */
  public static function from(array $data) {
    $variantData = optionalArrayData($data, 'variant');
    $variant = null;
    if ($variantData !== null) {
      $identityOnly = join('', array_keys($variantData)) === WinoApiVariantIdentity::fieldsQuery();
      $variant = $identityOnly ? WinoApiVariantIdentity::from($variantData) : WinoApiVariant::from($variantData);
    }

    return new self(
      nonBlankStringData($data, 'id'),
      dateData($data, 'createdAt', TimeUnit::javascriptTimestampLike()),
      WinoApiShopIdentity::from(arrayData($data, 'shop')),
      $variant,
      stringData($data, 'kind'),
      intData($data, 'quantity'),
      optionalStringData($data, 'comment')
    );
  }

  public const NODE_QUERY_NAME = "StockActivity";
  public const NODE_QUERY_RESOLVER = "stockActivity";

  /** @return self */
  public static function queryById(WinoApiClient $api, string $id, bool $queryFullVariant = false) {
    return self::from($api->postGraphQLNodeByIdQuery(
      self::NODE_QUERY_NAME,
      "",
      self::NODE_QUERY_RESOLVER,
      "",
      $id,
      self::fieldsQuery($queryFullVariant),
      []
    ));
  }

  public const COLLECTION_QUERY_NAME = "VariantStockActivities";
  public const COLLECTION_QUERY_RESOLVER = "stockActivities";

  /**
   * @param array<string, mixed> $queryVars
   * @return WinoApiStockActivityQueryAllIterator
   */
  public static function queryAllIterator(
    WinoApiClient $api,
    string $variantId,
    bool $queryFullVariant,
    string $queryName = self::COLLECTION_QUERY_NAME,
    string $queryArgs = "",
    string $resolverArgs = "",
    string $variantResolverArgs = "",
    array $queryVars = []
  ) {
    return new WinoApiStockActivityQueryAllIterator(
      $api,
      GraphQLPaginatedQueryRoot::nodeById(
        WinoApiVariant::NODE_QUERY_RESOLVER,
        $variantResolverArgs,
        $variantId
      ),
      $queryName,
      $queryArgs,
      self::COLLECTION_QUERY_RESOLVER,
      $resolverArgs,
      self::fieldsQuery($queryFullVariant),
      $queryVars
    );
  }

  /**
   * Using `queryAllIterator` might be prefered in order to reduce
   * memory footprint. This is especially true if you expect lot of
   * items in the array which could lead to reach the php memory limit.
   *
   * @return WinoApiStockActivity[]
   */
  public static function queryAll(
    WinoApiClient $winoApi,
    string $variantId,
    bool $queryFullVariant = false
  ) {
    return self::queryAllIterator($winoApi, $variantId, $queryFullVariant)->queryAll();
  }
}

/** @return bool */
function doesStockActivityCreationQueryFullVariant() {
  return true;
}

/** @return WinoApiStockActivity */
function createStockActivity(
  WebDeviceCache $cache,
  WinoApiClient $api,
  WinoApiShopIdentity $shop,
  string $variantId,
  int $quantity,
  string $comment,
  string $kind
) {
  $shopId = $shop->id;
  $deviceId = $cache->queryWebDevice($api, $shop)->id;
  $inputs = WinoApiClient::joinArgs(
    "kind: \$kind",
    "quantity: \$quantity",
    "comment: \$comment",
    "variantId: \$variantId",
    "shopId: \$shopId",
    "deviceId: \$deviceId"
  );

  return WinoApiStockActivity::from($api->postGraphQLNodeMutationQuery(
    "CreateWooCommerceStockMovement",
    WinoApiClient::joinArgs(
      "\$kind: StockActivityKind!",
      "\$quantity: Int!",
      "\$comment: String",
      "\$variantId: String!",
      "\$shopId: String!",
      "\$deviceId: String!"
    ),
    "createStockActivity",
    "input: { $inputs }",
    WinoApiStockActivity::fieldsQuery(doesStockActivityCreationQueryFullVariant()),
    [
      'kind' => $kind,
      'quantity' => $quantity,
      'comment' => $comment,
      'variantId' => $variantId,
      'shopId' => $shopId,
      'deviceId' => $deviceId
    ]
  ));
}

/** @return WinoApiStockActivity */
function createVariantStockActivity(
  WebDeviceCache $cache,
  WinoApiClient $winoApi,
  WinoApiVariant $variant,
  int $quantity,
  string $comment,
  string $kind
) {
  return createStockActivity(
    $cache,
    $winoApi,
    $variant->shop,
    $variant->id,
    $quantity,
    $comment,
    $kind
  );
}
