<?php

namespace Wino;

/** @extends GraphQLPaginatedQueryEdgeNodeIterator<WinoApiTax> */
class WinoApiTaxQueryAllIterator extends GraphQLPaginatedQueryEdgeNodeIterator {
  /** @return WinoApiTax */
  public function currentObject() {
    return WinoApiTax::from($this->currentNode());
  }
}

class WinoApiTax {
  public WinoApiShopIdentity $shop;
  public const SHOP_KEY = 'shop';

  public string $id;
  public const ID_KEY = 'id';

  public string $name;
  public const NAME_KEY = 'name';

  public ?\DateTime $archivedAt;
  public const ARCHIVED_AT_KEY = 'archivedAt';

  public float $value;
  public const VALUE_KEY = 'value';

  public function __construct(
    WinoApiShopIdentity $shop,
    string $id,
    string $name,
    float $value,
    ?\DateTime $archivedAt
  ) {
    $this->shop = $shop;
    $this->id = $id;
    $this->name = $name;
    $this->value = $value;
    $this->archivedAt = $archivedAt;
  }

  /** @return string */
  public static function fieldsQuery() {
    return WinoApiClient::graphQLFieldsQuery(
      self::ID_KEY,
      self::NAME_KEY,
      self::VALUE_KEY,
      self::ARCHIVED_AT_KEY,
      WinoApiClient::graphQLNestedFieldsQuery(
        self::SHOP_KEY,
        WinoApiShopIdentity::FIELDS_QUERY
      )
    );
  }

  /** @return self */
  public static function from(array $data) {
    return new self(
      WinoApiShopIdentity::from(arrayData($data, self::SHOP_KEY)),
      nonBlankStringData($data, self::ID_KEY),
      nonBlankStringData($data, self::NAME_KEY),
      floatData($data, self::VALUE_KEY),
      optionalDateData($data, self::ARCHIVED_AT_KEY, TimeUnit::javascriptTimestampLike()),
    );
  }

  public const NODE_QUERY_NAME = "Tax";
  public const NODE_QUERY_RESOLVER = "tax";

  /** @return WinoApiTax */
  public static function queryById(WinoApiClient $api, string $id) {
    return self::from($api->postGraphQLNodeByIdQuery(
      self::NODE_QUERY_NAME,
      "",
      self::NODE_QUERY_RESOLVER,
      "",
      $id,
      self::fieldsQuery(),
      []
    ));
  }

  public const COLLECTION_QUERY_NAME = "Taxes";
  public const COLLECTION_QUERY_RESOLVER = "taxes";

  /**
   * @param array<string, mixed> $queryVars
   * @return WinoApiTaxQueryAllIterator
   */
  public static function queryAllIterator(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop,
    string $queryName = self::COLLECTION_QUERY_NAME,
    string $queryArgs = "",
    string $filterByArgs = "",
    string $resolverArgs = "",
    array $queryVars = []
  ) {
    return new WinoApiTaxQueryAllIterator(
      $winoApi,
      GraphQLPaginatedQueryRoot::none(),
      $queryName,
      WinoApiShopIdentity::queryArgs($shop, $queryArgs),
      self::COLLECTION_QUERY_RESOLVER,
      WinoApiShopIdentity::resolverArgs($shop, $filterByArgs, $resolverArgs),
      self::fieldsQuery(),
      WinoApiShopIdentity::queryVars($shop, $queryVars)
    );
  }

  /**
   * Using `queryAllIterator` might be prefered in order to reduce
   * memory footprint. This is especially true if you expect lot of
   * items in the array which could lead to reach the php memory limit.
   *
   * @return WinoApiTax[]
   */
  public static function queryAll(
    WinoApiClient $winoApi,
    ?WinoApiShopIdentity $shop
  ) {
    return self::queryAllIterator($winoApi, $shop)->queryAll();
  }

  /** @return bool */
  public function isArchived() {
    return $this->archivedAt !== null;
  }
}
