<?php

namespace Wino;

?>

<script type="text/javascript">
  function winoAjaxElementTag(element) {
    return !element ? '' : element.tagName.toLowerCase().trim();
  }

  function winoAjaxInputType(input) {
    return !input ? '' : input.type.toLowerCase().trim();
  }

  function winoAjaxIsTag(element, tags) {
    return tags.indexOf(winoAjaxElementTag(element)) >= 0
  }

  function winoAjaxInputLikeTags() {
    return ['input', 'select', 'textarea']
  }

  function winoAjaxButtonTags() {
    return ['button']
  }

  function winoAjaxFormTags() {
    return ['form']
  }

  function winoAjaxFormTriggerTags() {
    return [...winoAjaxInputLikeTags(), ...winoAjaxButtonTags()]
  }

  function winoAjaxIsInputLike(element) {
    return winoAjaxIsTag(element, winoAjaxInputLikeTags())
  }

  function winoAjaxIsButton(element) {
    return winoAjaxIsTag(element, winoAjaxButtonTags())
  }

  function winoAjaxIsForm(element) {
    return winoAjaxIsTag(element, winoAjaxFormTags())
  }

  function winoAjaxIsFormTrigger(element) {
    return winoAjaxIsTag(element, winoAjaxFormTriggerTags())
  }

  function winoAjaxInputValue(input) {
    if(!winoAjaxIsInputLike(input)) return undefined
    const tag = winoAjaxElementTag(input);
    const type = winoAjaxInputType(input);
    const kind = `${tag}-${type}`;
    if (kind === 'input-checkbox') return input.checked;
    return input.value;
  }

  function winoReplaceDomById(id, html) {
    const node = document.getElementById(id);
    if (!node) return
    const div = document.createElement('div');
    div.innerHTML = html;
    node.parentNode.replaceChild(div.firstChild, node);
  }

  function winoAjaxData() {
    const pluginVersionCheckKey = <?= jsonString(FormHandling::pluginVersionCheckKey()) ?>;
    const pluginVersionCheckValue = <?= jsonString(FormHandling::pluginVersionCheckValue()) ?>;

    const options = <?= FormHandling::stringifyInputNamesArray() ?>;

    const optionsData = {};
    for (const option of options) {
      const input = document.getElementById(option);
      const value = winoAjaxInputValue(input);
      if (value === undefined) continue;
      optionsData[option] = value;
    }
    optionsData[pluginVersionCheckKey] = pluginVersionCheckValue;

    return optionsData;
  }

  function winoAjaxRemoveDivById(divId) {
    const div = document.getElementById(divId);
    if(div) div.parentElement.removeChild(div);
  }

  function winoAjaxDismissibleNoticeId() {
    let counter = winoAjaxDismissibleNoticeId.idCounter;
    if(typeof counter !== 'number') counter = 0
    winoAjaxDismissibleNoticeId.idCounter = counter + 1;
    return 'wino-ajax-dismissible-notice-' + counter;
  }

  function winoAjaxDismissCodeClass(code) {
    return `notice-code-${code}`
  }

  function winoAjaxDismissNotice(code) {
    const rootClass = winoAjaxDismissCodeClass(code)
    const detailsClass = `details-${rootClass}`
    const noticeRoots = document.querySelectorAll(`.${rootClass}`)
    for (const notice of noticeRoots) notice.parentElement.removeChild(notice)
    const details = document.querySelectorAll(`.${detailsClass}`)
    for (const detail of details) {
      let notice = detail.parentElement;
      if (!notice.classList.contains('notice')) continue
      notice.parentElement.removeChild(notice)
    }
  }

  function winoAjaxDismissibleNotice(content, type, code) {
    const div = document.createElement('div');
    const id = winoAjaxDismissibleNoticeId();
    div.id = id;
    if(code) { div.classList.add(winoAjaxDismissCodeClass(code)) }
    if(typeof type === 'string') div.classList.add('notice', 'notice-' + type, 'is-dismissible');
    else div.classList.add('notice', 'is-dismissible');

    const contentParagraph = document.createElement('p')
    contentParagraph.innerHTML = content.replace(/\\n/g, '<br />')

    const button = document.createElement('button')
    button.type = 'button'
    button.classList.add('notice-dismiss')
    button.innerHTML = '<span class="screen-reader-text">Dismiss this notice.</span>';
    div.insertAdjacentElement('beforeend', contentParagraph)
    div.insertAdjacentElement('beforeend', button)

    const event = 'click'
    const buttonEventListener = () => {
      button.removeEventListener(event, buttonEventListener)
      winoAjaxRemoveDivById(id)
    }
    button.addEventListener(event, buttonEventListener)
  
    return div;
  }

  function winoAjaxSummaryNotice(type, summary, details, code) {
    return winoAjaxDismissibleNotice(
      '<details><summary>' + summary + '</summary>' + details + '</details>',
      type,
      code
    );
  }

  function winoAjaxErrorMessage(error) {
    if (error === null || error === undefined) return ''
    if (typeof error === 'string') return error
    if ('message' in error) return winoAjaxErrorMessage(error.message)
    return ''
  }

  function winoAjaxErrorNotice(error, errorCode) {
    const errorMessage = winoAjaxErrorMessage(error)
    let summary = errorMessage;
    let details = '';
    const splitIndex = errorMessage.indexOf(' in ');
    if(splitIndex >= 0) {
      summary = errorMessage.substring(0, splitIndex);
      const errorStack = errorMessage.substring(splitIndex);
      details = '<i style="word-break:break-word">' + errorStack + '</i>';
    }
    return details === ''
      ? winoAjaxDismissibleNotice(summary, 'error', errorCode)
      : winoAjaxSummaryNotice('error', summary, details, errorCode);
  }

  function winoAjaxScrollTop() {
    const scroll = window['scrollTo']
    if(typeof scroll !== 'function') return
    scroll({
      top: 0,
      behavior: 'smooth'
    })
  }

  function winoAjaxInsertNotice(noticeElement) {
    const anchorClass = "wino-ajax-notice-anchor"
    let anchorDiv = document.querySelector(`.${anchorClass}`)
    if(!anchorDiv) {
      let anchorRoot = document.querySelector('form')
      if(!anchorRoot) anchorRoot = document.querySelector('.wrap > h1 + div')
      if(!anchorRoot) throw new Error('Missing notice anchor')
      anchorDiv = document.createElement('div');
      anchorDiv.classList.add(anchorClass)
      anchorRoot.insertAdjacentElement('afterbegin', anchorDiv);
    }
    winoAjaxScrollTop()
    anchorDiv.insertAdjacentElement('afterend', noticeElement)
  }

  function winoAjaxActionTriggers(action) {
    const genericActionClass = "<?= ajaxActionClass(); ?>";
    const actionClass = `${genericActionClass}-${action}`
    const actionClassElements = document.querySelectorAll(`.${actionClass}`);
    const triggers = [];
    for(const element of actionClassElements) triggers.push(element);
    return triggers;
  }

  function winoAjaxUpdateActionButtonLabel(action, label) {
    if(typeof label !== 'string') return;
    const trim = label.trim()
    if(trim === '') return
    const triggers = winoAjaxActionTriggers(action);
    for(const trigger of triggers)
    if(!winoAjaxIsButton(trigger)) continue
    else trigger.textContent = trim
  }

  function winoAjaxCanBeDisabled(trigger) {
    const { classList } = trigger
    const prevented = classList.contains('disabling-prevented')
    const noticeDismiss = classList.contains('notice-dismiss')
    return !prevented && !noticeDismiss;
  }

  function winoAjaxCanBeEnabled(trigger) {
    return !trigger.classList.contains('protected')
  }

  function winoAjaxEnableTrigger(trigger) {
    if (!winoAjaxCanBeEnabled(trigger)) return;
    trigger.classList.remove('disabled')
    if (winoAjaxIsFormTrigger(trigger)) trigger.disabled = false
    if (!winoAjaxIsForm(trigger)) return
    const inputs = trigger.querySelectorAll(winoAjaxFormTriggerTags().join(', '))
    for(const input of inputs) winoAjaxEnableTrigger(input)
  }

  function winoAjaxDisableTrigger(trigger) {
    if (!winoAjaxCanBeDisabled(trigger)) return;
    trigger.classList.add('disabled')
    if (winoAjaxIsFormTrigger(trigger)) trigger.disabled = true
    if (!winoAjaxIsForm(trigger)) return
    const inputs = trigger.querySelectorAll(winoAjaxFormTriggerTags().join(', '))
    for(const input of inputs) winoAjaxDisableTrigger(input)
  }

  function winoAjaxEnableAction(action, label) {
    winoAjaxUpdateActionButtonLabel(action, label)
    const triggers = winoAjaxActionTriggers(action);
    triggers.forEach(winoAjaxEnableTrigger)
  }

  function winoAjaxDisableAction(action, label) {
    const triggers = winoAjaxActionTriggers(action);
    triggers.forEach(winoAjaxDisableTrigger)
    winoAjaxUpdateActionButtonLabel(action, label)
  }

  function winoAjaxHasActionClass(action, className) {
    const triggers = winoAjaxActionTriggers(action);
    return triggers.some(trigger => trigger.classList.contains(className))
  }

  function winoAjaxIsActionEnabled(action) {
    const disabled = winoAjaxHasActionClass(action, 'disabled')
    if(disabled) winoAjaxDisableAction(action)
    return !disabled
  }

  function winoAjaxIsActionDisabledAfterFirstUsage(action) {
    return winoAjaxHasActionClass(action, 'disable-after-first-usage')
  }

  function winoDelay(duration) {
    return new Promise(resolve => setTimeout(() => resolve(), duration))
  }

  function winoTimeout(handler, timeout) {
    return winoDelay(timeout).then(() => handler())
  }

  function winoUnavailableServerError(err) {
    return new Error(
      `Le serveur ne semble pas disponible ${err}`
    )
  }

  function winoContactTeamMessage(reason) {
    return reason + ' ' +
    'Si le problème persiste, ' +
    '<a href="http://help.wino.fr/" target="_blank">contactez les équipes Wino.</a>';
  }

  function winoAjaxPostJsonFromResponse(response, data, retry, retryInterval) {
    const maxRetry = 8;
    const nextRetry = retry + 1
    const nextRetryInterval = Math.trunc(retryInterval * 1.5)

    return response.json().catch(error => {
      if (retry < maxRetry) return winoTimeout(
        () => winoAjaxPost(data, nextRetry, nextRetryInterval),
        retryInterval
      )

      const message = 'Le serveur WordPress semble injoignable pour le moment.'
      const tryRefresh = 'Essayez de rafraîchir la page dans quelques instants.'

      winoAjaxInsertNotice(winoAjaxSummaryNotice(
        'error',
        winoContactTeamMessage(message + ' ' + tryRefresh),
        error
      ));

      return null;
    })
  }

  function winoAjaxPost(data, retry = 0, retryInterval = 1000) {
    return fetch(ajaxurl, {
      method: 'POST',
      body: data
    })
    .then(response => winoAjaxPostJsonFromResponse(response, data, retry, retryInterval))
  }

  function winoAjaxThrowErrorHandler(error) {
    winoAjaxInsertNotice(winoAjaxSummaryNotice(
      'error',
      winoContactTeamMessage('Une erreur est survenue. Veuillez réessayer.'),
      error
    ))
  }

  function winoAjaxNoticeErrorHandler(error, errorCode) {
    winoAjaxInsertNotice(winoAjaxErrorNotice(error, errorCode))
  }

  function winoAjaxConsoleErrorHandler(error) {
    console.log(error)
  }

  function winoAjaxErrorHandler(name, inheritedHandler) {
    if (name === 'throw') return winoAjaxThrowErrorHandler
    if (name === 'notice') return winoAjaxNoticeErrorHandler
    if (name === 'console') return winoAjaxConsoleErrorHandler
    return typeof inheritedHandler === 'function' ? inheritedHandler : winoAjaxThrowErrorHandler;
  }

  function winoAjaxCallAction(action, errorHandlerName) {
    let errorsCount = 0;
    const data = new FormData();
    const rootErrorHandler = winoAjaxErrorHandler(errorHandlerName)

    data.append('action', action);
    const additionalData = winoAjaxData();
    for (const key in additionalData) data.append(key, additionalData[key]);

    return winoAjaxPost(data).then((data) => {
      const c = data ? data.length : 0;
      let i, div;

      for (i = 0; i < c; i++) {
        const dataEntry = data[i];
        const type = dataEntry['type'];

        if (type === "dom-replacement-by-id") {
          winoReplaceDomById(dataEntry.id, dataEntry.html);
          continue;
        }

        if (type === "enable-action") {
          winoAjaxEnableAction(dataEntry.action, dataEntry.label);
          continue;
        }

        if (type === "disable-action") {
          winoAjaxDisableAction(dataEntry.action, dataEntry.label);
          continue;
        }

        if (type === "call-action") {
          winoAjaxCallAction(dataEntry.action, dataEntry.errorHandlerName);
          continue;
        }

        if (type === "dismiss-notice") {
          winoAjaxDismissNotice(dataEntry.code);
          continue;
        }

        const message = '' + dataEntry['msg'];
        if (type !== 'error') winoAjaxInsertNotice(winoAjaxDismissibleNotice(message, type))
        else {
          winoAjaxErrorHandler(dataEntry['errorHandlerName'], rootErrorHandler)(message, dataEntry['errorCode'])
          errorsCount++
        }
      }
      return errorsCount
    }).catch(error => {
      rootErrorHandler(error)
      return errorsCount + 1
    });
  }

  function winoAjaxForce(force, action) {
    if (!winoAjaxIsActionEnabled(action) && !force) return;

    const disableAfterFirstUse = winoAjaxIsActionDisabledAfterFirstUsage(action)
    if (disableAfterFirstUse) winoAjaxDisableAction(action);

    winoAjaxCallAction(action).then(errorsCount => {
      if (disableAfterFirstUse && errorsCount > 0) winoAjaxEnableAction(action) 
    })
  }

  function winoAjaxCallActionOnClick(action) {
    winoAjaxForce(false, action)
  }

  function winoAjaxOnChange(id) {
    errorHandler = winoAjaxErrorHandler();
    const input = document.getElementById(id)
    if (!input) return errorHandler(`Missing input (id=${id})`);
    let form = input.parentElement
    while(!winoAjaxIsForm(form)) form = form.parentElement
    if (!form) return errorHandler(`Missing form for input (id=${id})`);
    const action = form.getAttribute('action')
    if (!action) return errorHandler(`Missing form action for input (id=${id})`);
    winoAjaxCallActionOnClick(action)
  }

  function winoAjaxRunStartUpActions() {
    const actionElements = document.querySelectorAll('.wino-start-up-action')
    for(const actionElement of actionElements) {
      const action = actionElement.textContent.trim()
      winoAjaxForce(true, action)
    }
  }
  
  window.addEventListener("load", () => {
    winoAjaxRunStartUpActions()
  });
</script>
